@@ -3,7 +3,7 @@
  * Copyright (C) 2002 The gtkmm Development Team
  *
  * Authors: Federico Mena-Quintero <federico@gimp.org>
- * gtkmm port: Daniel Elstner <daniel.elstner@gmx.net>
+ * gtkmm port: Daniel Elstner <daniel.kitta@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,39 +37,35 @@
 #include <gtkmm/style.h>
 #include <gtkmm/window.h>
 
-#ifndef M_PI
-#define M_PI 3.14159265359
-#endif /* M_PI */
-
 namespace
 {
 
-/* These values control the smoothness and speed of the animation.
+/*
+ * These values control the smoothness and speed of the animation.
  * For instance, FRAME_DELAY=25 and CYCLE_LEN=90 looks much better
  * to me (and also wastes more CPU cycles, of course).
  */
 enum
 {
-  FRAME_DELAY = 50,
-  CYCLE_LEN   = 60
+  FRAME_DELAY = 48,
+  CYCLE_LEN   = 64
 };
 
-const char background_name[] =
-  "background.jpg";
+const char * const background_name =
+  "gtk-demo/background.jpg";
 
 const char * const image_names[] =
 {
-  "apple-red.png",
-  "gnome-applets.png",
-  "gnome-calendar.png",
-  "gnome-foot.png",
-  "gnome-gmush.png",
-  "gnome-gimp.png",
-  "gnome-gsame.png",
-  "gnu-keys.png"
+  "gtk-demo/apple-red.png",
+  "gtk-demo/gnome-applets.png",
+  "gtk-demo/gnome-calendar.png",
+  "gtk-demo/gnome-foot.png",
+  "gtk-demo/gnome-gmush.png",
+  "gtk-demo/gnome-gimp.png",
+  "gtk-demo/gnome-gsame.png",
+  "gtk-demo/gnu-keys.png"
 };
 
-
 class DemoRenderArea : public Gtk::DrawingArea
 {
 public:
@@ -84,25 +80,41 @@ private:
   std::vector< Glib::RefPtr<const Gdk::Pixbuf> >  images_;
   Glib::RefPtr<Gdk::Pixbuf>                       current_frame_;
   unsigned int                                    frame_num_;
-  sigc::connection                                conn_timeout_;
 
-  void generate_next_frame_();
+  void generate_next_frame();
 };
 
+Glib::RefPtr<Gdk::Pixbuf> create_pixbuf(const std::string& name)
+{
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  return Gdk::Pixbuf::create_from_file(name);
+#else
+  // Ignore this; it's broken by design.
+  std::auto_ptr<Glib::Error> error;
+  return Gdk::Pixbuf::create_from_file(name, error);
+#endif
+}
 
-/* Load all image files, create an empty buffer for storing the current frame,
+/*
+ * Load all image files, create an empty buffer for storing the current frame,
  * and install a timeout handler that will be called periodically.  The show
  * will start as soon as Gtk::Main::run() is invoked.
  */
 DemoRenderArea::DemoRenderArea()
 :
-  background_ (Gdk::Pixbuf::create_from_file(background_name)),
-  frame_num_  (0)
+  frame_num_ (0)
 {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  background_ = Gdk::Pixbuf::create_from_file(background_name);
+#else
+  // Ignore this; it's broken by design.
+  std::auto_ptr<Glib::Error> error;
+  background_ = Gdk::Pixbuf::create_from_file(background_name, error);
+#endif
   std::transform(
       &image_names[0], &image_names[G_N_ELEMENTS(image_names)],
       std::back_inserter(images_),
-      &Gdk::Pixbuf::create_from_file);
+      &create_pixbuf);
 
   const int back_width  = background_->get_width();
   const int back_height = background_->get_height();
@@ -113,17 +125,16 @@ DemoRenderArea::DemoRenderArea()
   set_size_request(back_width, back_height);
   add_events(Gdk::EXPOSURE_MASK);
 
-  conn_timeout_ = Glib::signal_timeout().connect(
-      sigc::bind_return(sigc::mem_fun(*this, &DemoRenderArea::generate_next_frame_), true),
+  Glib::signal_timeout().connect(
+      sigc::bind_return(sigc::mem_fun(*this, &DemoRenderArea::generate_next_frame), true),
       FRAME_DELAY);
 }
 
 DemoRenderArea::~DemoRenderArea()
-{
-  conn_timeout_.disconnect();
-}
+{}
 
-/* Expose event handler of the widget.  Just fill the exposed
+/*
+ * Expose event handler of the widget.  Just fill the exposed
  * area with the corresponding pixmap data from current_frame_.
  */
 bool DemoRenderArea::on_expose_event(GdkEventExpose* event)
@@ -141,10 +152,11 @@ bool DemoRenderArea::on_expose_event(GdkEventExpose* event)
   return true; // stop signal emission
 }
 
-/* Generate the next frame of the animation and store it into current_frame_.
+/*
+ * Generate the next frame of the animation and store it into current_frame_.
  * The expose_event handler accesses that buffer to do the actual drawing.
  */
-void DemoRenderArea::generate_next_frame_()
+void DemoRenderArea::generate_next_frame()
 {
   const int back_width  = background_->get_width();
   const int back_height = background_->get_height();
@@ -154,14 +166,14 @@ void DemoRenderArea::generate_next_frame_()
   const double xmid = back_width  / 2.0;
   const double ymid = back_height / 2.0;
 
-  const double f = 2.0 * M_PI * double(frame_num_) / CYCLE_LEN;
+  const double f = 2.0 * G_PI * double(frame_num_) / CYCLE_LEN;
 
   double r = std::min(xmid, ymid) / 2.0;
   r += (r / 3.0) * std::sin(f);
 
   for(unsigned i = 0; i < images_.size(); ++i)
   {
-    const double ang = 2.0 * M_PI * double(i) / images_.size() - f;
+    const double ang = 2.0 * G_PI * double(i) / images_.size() - f;
 
     const double iw = images_[i]->get_width();
     const double ih = images_[i]->get_height();
@@ -169,7 +181,7 @@ void DemoRenderArea::generate_next_frame_()
     const int xpos = int(std::floor(xmid + r * std::cos(ang) - iw / 2.0 + 0.5));
     const int ypos = int(std::floor(ymid + r * std::sin(ang) - ih / 2.0 + 0.5));
 
-    const double depth = (i & 1) ? std::sin(f) : std::cos(f);
+    const double depth = ((i % 2) != 0) ? std::sin(f) : std::cos(f);
     const double scale = std::max(0.25, 2.0 * depth * depth);
 
     Gdk::Rectangle rect (xpos, ypos, int(iw * scale), int(ih * scale));
@@ -177,7 +189,7 @@ void DemoRenderArea::generate_next_frame_()
 
     if(!rect.has_zero_area())
     {
-      const int overall_alpha = std::max(127, std::abs(int(255.0 * depth)));
+      const int overall_alpha = std::max(127, int(std::abs(255.0 * depth)));
 
       images_[i]->composite(
           current_frame_,
@@ -187,7 +199,6 @@ void DemoRenderArea::generate_next_frame_()
           overall_alpha);
     }
   }
-
   frame_num_ = (frame_num_ + 1) % CYCLE_LEN;
 
   // Tell GTK+ the widget should be redrawn soon.  This will trigger the
@@ -197,10 +208,11 @@ void DemoRenderArea::generate_next_frame_()
 
 } // anonymous namespace
 
-
 int main(int argc, char** argv)
 {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
+#endif
   {
     Gtk::Main main_instance (&argc, &argv);
 
@@ -212,12 +224,13 @@ int main(int argc, char** argv)
 
     Gtk::Main::run(window);
   }
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   catch(const Glib::Error& error)
   {
     std::cerr << error.what() << std::endl;
     return EXIT_FAILURE;
   }
-
+#endif
   return EXIT_SUCCESS;
 }
 
